<?php

/**
 * @file bt_tracker.admin.inc
 *   Provide the admin interface for the bittorrent tracker module
 */

/**
 * Define BitTorrent Tracker settings
 *
 * @return The administrative form
 */
function bt_tracker_admin_settings() {
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General Tracker Settings'),
    '#description' => t('These are general settings for the BitTorrent Tracker. Mostly naming your tracker and setting the privacy scope.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['general']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Tracker Name'),
    '#description' => t('This is the name of the BitTorrent Tracker.'),
  );
  $form['general']['enabled'] = array(
    '#type' => 'radios',
    '#title' => t('Tracker Status'),
    '#options' => array(
      t('Offline'),
      t('Online'),
    ),
    '#description' => t('Determines whether the tracker is online of off.'),
  );
  $form['general']['scope'] = array(
    '#type' => 'radios',
    '#title' => t('Tracker Scope'),
    '#options' => array(
      t('Public'),
      t('Mixed'),
      t('Private'),
    ),
    '#description' => t('Sets the scope of the tracker. Public allows for the best performance and complete anonymity, but at the cost of statistics. Mixed tracks statistics of registered users and still allows anonymous access. Private restricts access to registered users and has the most fine grained control.'),
  );

  $form['intervals'] = array(
    '#type' => 'fieldset',
    '#title' => t('Tracker Intervals'),
    '#description' => t('These settings control how often certain events occur within the tracker. This includes the announce and scrape intervals for the client as well as how often the tracker should auto-prune the active users information. Please note that how often the code is executed depends on when your cron jobs run'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['intervals']['announce_interval'] = array(
    '#type' => 'textfield',
    '#title' => t('Announce Interval'),
    '#description' => t('This is how often the torrent clients perform announce requests to the tracker.'),
  );
  $form['intervals']['scrape_interval'] = array(
    '#type' => 'textfield',
    '#title' => t('Scrape Interval'),
    '#description' => t('This is how often the torrent clients perform scrape requests to the tracker.'),
  );
  $form['intervals']['prune_interval'] = array(
    '#type' => 'textfield',
    '#title' => t('Auto-prune Interval'),
    '#description' => t('Determines when entries in the active users table will be automatically removed. Please note that this is also dependent on how often cron is run.'),
  );
  $form['intervals']['stats_interval'] = array(
    '#type' => 'textfield',
    '#title' => t('Statistics Update Interval'),
    '#description' => t('Determines how often the statistics for each torrent are regenerated based off of the active users table.'),
  );

  $form['limits'] = array(
    '#type' => 'fieldset',
    '#title' => t('Tracker use requirements'),
    '#description' => t('These settings determine whether or not a user retrieves a result from the tracker.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['limits']['maximum_simultaneous_downloads'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum number of simultaneous downloads'),
    '#description' => t('This is how many downloads a user may have running simultaneously (0 for unlimited).'),
  );
  $form['limits']['minimum_ratio'] = array(
    '#type' => 'textfield',
    '#title' => t('Minimum sharing ratio'),
    '#description' => t('This is the minimum ratio a user must have to recieve a response from the tracker (0 for unlimited).'),
  );

  /*$form['web-seeding'] = array(
    '#type' => 'fieldset',
    '#title' => t('Web Seeding'),
    '#description' => t('Web seeding allows the web server to act as a torrent client providing pieces of a download should no seeds be available. There are two implementation for web seeding. The "GetRight" implmentation is very simple to use, the torrent client queries the server for pieces of the file. A caveat is that there is no limiting how often this is used and who may use it. The second implmentation was proposed by "BitTornado" and it allows finer control over who may access files and when they may do so.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['web-seeding']['web_seeding'] = array(
    '#type' => 'radios',
    '#title'  => t('Enable Web Seeding'),
    '#options' => array(
      t('Disabled'),
      t('Enabled - GetRight'),
      t('Enabled - BitTornado'),
      t('Enabled - Both'),
    ),
    '#description' => t('This determines whether or not the tracker will support web-seeding. The GetRight method is less secure and may be abused. By contrast the BitTornado method is more secure, but requires more CPU usage as all data is passed through PHP.'),
  );*/

  return system_settings_form(setting_prepare_settings_form('bt_tracker', $form));
}