<?php

/**
 * @file
 * The scrape portion of the tracker module.
 *
 * Accepts the input from the users torrent client
 * and acts accordingly.
 */

// Require the bt functions
require_once('bt_common.inc');

// Include bootstrap.inc and run the bootstrap
require_once('includes/bootstrap.inc');
drupal_bootstrap(DRUPAL_BOOTSTRAP_DATABASE);

// Include "module.inc"
require_once('includes/module.inc');

// Include settings functions
require_once('module_settings.inc');

// Include "bt_tracker" module settings
module_load_include('inc', 'bt_tracker', 'bt_tracker.settings');

// Initializes $conf so we can use variable_get & setting_get
$conf = variable_init(isset($conf) ? $conf : array());

// Do not continue parsing if the tracker is offline
if (setting_get('bt_tracker', 'enabled') == 1) {
  $response = array('files' => array());
  $hashes = array();

  $request_variables = split('&', $_SERVER['QUERY_STRING']);
  foreach ($request_variables as &$request_var) {
    preg_match('/info_hash=/', $request_var, $matches);

    if (count($matches) == 0 ) {
      unset($request_var);
    }
    else {
      // Strip 'info_hash=' from the begining of the value and urldecode it.
      $hashes[] = bin2hex(urldecode(substr($request_var, 10)));
    }
  }

  if (count($hashes[0]) == 0) {
    bt_error('Please supply an info_hash');
  }

  foreach ($hashes as $info_hash) {
    $valid = db_result(db_query("SELECT fid FROM {bt_torrents} WHERE info_hash = '%s'", $info_hash));
    if ($valid) {
      $response['files'][$info_hash] = module_invoke('bt_tracker', 'scrape', $info_hash);
    }
  }

  if (count($response['files']) == 0) {
    bt_error('Invalid info hash(s)');
  }

  $response['flags'] = array();
  $response['flags']['min_request_interval'] = setting_get('bt_tracker', 'scrape_interval');

  // Loop through the files array updating info_hash keys into their correct binary strings
  $files = $response['files'];
  unset($response['files']);
  $response['files'] = array();
  foreach ($files as $info_hash => $scrape_info) {
    $response['files'][pack('H*', $info_hash)] = $scrape_info;
  }

  // Return the response
  bencode_response_raw(bencode($response));
}
else {
  // Let the client know that the tracker is offline.
  bt_error('Tracker is Offline');
}